@** The daemon.

This is the daemon that runs with superuser privileges. It handles all
the communication over the USB bus, and multiplexes the connections from
applications trying to access smart cards.
@c

@<System includes@>@;
@<Local includes@>@;
@<Function declarations@>@;
@<Global variables@>@;

int main(int argc, char** args)
{
   assert (true, "Assert is broken");
   debug("Starting");
   
   read_arguments(argc, args); /* read arguments */

   if (check_running(param.detach)) { /* check for running copy, optionally demonize */

      if (!send_args()) { // already running, send message
         printf("Error: PID file exists but ");
         @<Check error on connecting to daemon@>;
      }
      return 0;
   }

   @<Log startup details to syslog@>;

   @<Create \& bind to sockets@>;
 
   initialise_usb(); /* initialise usb connections */

   listen_sockets(); /* start listening on the sockets */
  
   @<Perform clean up operations@>;

   return 0;
}

@ Log startup to syslog.
We setup if we are logging to syslog or stderr, then write the initial state to the log.
@<Log startup...@>=
   setup_syslog(param.syslog); // setup if we log to syslog or to stderr
   slog("Daemon starting");
   slog("");
   slog(" -- Options --");
   slog("program: %s", NULL==param.program?"":param.program);
   slog("config file: %s", NULL==param.config?"":param.config);
   slog("daemonize: %s", param.detach?"true":"false");
   slog("syslog: %s", param.syslog?"true":"false");
   slog("");

@ Check error from connecting.
If we are trying to send a message to another daemon and get an error,
try and diagnose it.
@<Check error on con...@>=
   switch (errno) {
      case EACCES:
      case EPERM:
         printf("cannot connect to daemon: Permission Denied - are you root?\n");
         break;
      case ECONNREFUSED:
         printf("cannot connect to daemon, not running\n");
         break;
      default:
         printf("failed to connect to daemon\n");
   }

@ Create \& bind to sockets.
Make directory and file descriptor for sockets
@<Create \&...@>=
   debug("Creating sockets");
   setup_sockets(&master_socket, (struct sockaddr*) &master_info, "master", 
                 S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH | S_IWOTH | S_IWGRP);
   setup_sockets(&control_socket, (struct sockaddr*) &control_info, "control", S_IRUSR | S_IWUSR);

@ Cleaning up our sockets and pidfile.
We need to close and unlink the sockets and remove the pid file.
@<Perform clean...@>=
   debug("Cleaning up");
   clean_up(master_socket, (struct sockaddr*) &master_info);
   clean_up(control_socket, (struct sockaddr*) &control_info);
   assert (unlink(pidfile) == 0, "Failed to clean up - couldn't unlink pidfile");


@ System library includes.
We include here the system libraries for socket and filesystem IO 
and string and error handling. We also need some types exported from
lib-USB.
@<System includes...@>=
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <unistd.h>
#include <usb.h>

@ Local header includes.
These import the USB IO functions, the protocol layer and debugging functions.
@<Local includes...@>=
#include "usb.h"
#include "structs.h"
#include "../share/types.h"
#include "../share/debug.h"
#include "../share/log.h"
#include "../share/protocol.h"

@ Global Variables.
@<Global...@>=
#define MAXCONNQUEUE 5

int master_socket, control_socket;
// the sockets to listen(2) on

struct sockaddr_struct {
   sa_family_t family;
   char sa_data[MAXSOCKETPATH];
} master_info, control_info;
/* the info on the sockets 
 (struct sockaddr extended to fit 
 the path length in) */

pclient client_root = NULL;
// a pointer to the linked list of currently connected clients

char pidfile[20];
// the name of the pidfile to write


@* Program Options.

The daemon takes several command line options. 
The functions to parse them are given here as well.

\halign {\hfil \it # & # & # \hfil \cr
   {\bf short} & {\bf long} & {\bf definition} \cr
   -c & --config & Specify Config File \cr
   -d & --no-detach & Don't leave console \cr
   -s & --no-syslog & Log to stderr not syslog \cr
   -f & --force & Force starting of the daemon \cr
   -h & --help & Show this help \cr
   -m & -message & Control message to send to existing process \cr
}

@ Options structure.
The program options are all stored in this structure.
@<Global...@>=
struct param_struct {
   char* message;
   char* config;
   char* program;
   bool syslog;
   bool detach;
   bool force;
} param;

@ Procedure |print_syntax()|.
Prints out the program syntax
@c
void print_syntax()
{
   printf("Syntax: cardd <options>\n");
   printf("Options:\n");
   printf("\t-c <file>\n\t--config <file>\t\tSpecify Config File\n");
   printf("\t-d\n\t--no-detach\t\tDon't leave console\n");
   printf("\t-s\n\t--no-syslog\t\tLog to stderr not syslog\n");
   printf("\t-f\n\t--force\t\tForce starting of the daemon\n");
   printf("\t-h\n\t--help\t\t\tShow this help\n");
   printf("\t-m <message>\n\t--message <message>\tControl message to send to existing process\n");
   printf("Messages:\n");
   printf("\tquit\texit the running daemon\n");
   printf("\tconfig\tre-read the config file\n");
}

@ Procedure |read_arguments()|.
Parses all the arguments passed to the program into param structure

\halign{\hfil \it # & # \hfil \cr
|argc|& The number of arguments\cr
|args|& An array of strings of the arguments\cr
}
@c
void read_arguments(int argc, char** args)
{
   @<Initialise the |param| structure@>;
   
   param.program = (*args); // the name the program was called as
   args++;
   argc--;
   
   while (argc > 0) { /* While there are more arguments */
      debug ("Parameter: %s", *args);
      @<Put the paramter into the structure@>;
      args++;
      argc--; // go onto next one
   }
}

@ 
@<Initialise the |param|...@>=
   param.message = NULL;
   param.config = NULL;
   param.syslog = true;
   param.detach = true;

@ 
@<Function...@>=
void read_arguments(int argc, char** args);

@ Check the parameter.
This code takes the parameter and puts it into the |param| structure.
If the parameter takes an argument, it check that this exists and sets
its value in the structure. If an error occurs, we print the program
syntax and exit
@<Put the param...@>=
      if (0 == strcmp(*args, "-c") || 0 == strcmp(*args, "--config")) {
         argc--;
         if (0 >= argc) { print_syntax(); exit(0); } // not enough arguments
         args++;
         param.config = (*args);
         debug("Got parameter config: %s", param.config);

      } else if (0 == strcmp(*args, "-d") || 0 == strcmp(*args, "--no-detach")) {
         param.detach = false;
         debug("Got parameter no-detach");

      } else if (0 == strcmp(*args, "-s") || 0 == strcmp(*args, "--no-syslog")) {
         param.syslog = false;
         debug("Got parameter no-syslog");

      } else if (0 == strcmp(*args, "-f") || 0 == strcmp(*args, "--force")) {
         param.force = true;
         debug("Got parameter force");

      } else if (0 == strcmp(*args, "-h") || 0 == strcmp(*args, "--help")) {
         print_syntax();
         exit(0);

      } else if (0 == strcmp(*args, "-m") || 0 == strcmp(*args, "--message")) {
         argc--;
         if (0 >= argc) { print_syntax(); exit(0); } // not enough arguments
         args++;
         param.message = (*args);
         debug("Got parameter message: %s", param.message);

      } else {
         // unknown argument
         print_syntax();
         exit(0);
      }

@* Function |build_fd_list()|.
Takes all the clients in a linked list , and adds them to an |fd_set|. 
The function returns the highest fd + 1.

\halign{\hfil \it # & # \hfil \cr
|fds|& The |fd_set| to add the clients to\cr
|root|& The root of the linked list to traverse\cr
|scount|& One more than the highest current fd in the |fd_set|\cr
}
@c
int build_fd_list(fd_set* fds, pclient root, int scount)
{
   pclient current = root;
   while (NULL != current) { // for all the clients in the list
      FD_SET(current->socket, fds); // add this one to the set
      debug("current->socket: %d", current->socket);
      if (scount <= current->socket)
         scount = current->socket + 1; // increment the count if neccessary
      debug("scount: %d", scount);
      current = current->next; // go onto next
   }
   return scount; // return the count
}

@ 
@<Function...@>=
int build_fd_list(fd_set* fds, pclient root, int scount);

@* Procedure |setup_sockets()|.
Binds to a socket with |bind(2)| and |listen(2)| on it. Here we also check the
directory the sockets are in, and set the permissions on it
\halign{\hfil \it # & # \hfil \cr
|skt|& The socket file descriptor\cr
|sktinfo|& The socket info structure\cr
|file|& The filename (|SOCKETDIR"/"| will be prepended) to bind to.\cr
|perms|& The permissions of the socket to set\cr
}

@c
void setup_sockets(int* skt, struct sockaddr* sktinfo, char* file, mode_t perms)
{
   socklen_t len;
   int ret;

   mkdir(SOCKETDIR, S_ISVTX | S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
   chmod(SOCKETDIR, S_ISVTX | S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
   // make the socket directory and set its permissions
 
   *skt = socket (AF_UNIX, SOCK_STREAM, 0); // create a UNIX socket
   assert(0 <= *skt, "Can't get a socket fd");
   debug("Creating socket %d", *skt);

   sktinfo->sa_family = AF_UNIX;
   strcpy(sktinfo->sa_data, SOCKETDIR);
   strcat(sktinfo->sa_data, "/");
   strcat(sktinfo->sa_data, file);
   // setup the info structure
  
   ret = unlink(sktinfo->sa_data);
   assert (ret >= 0 || errno == ENOENT, "Can't unlink socket");
   // make sure we can remove it before binding to it

   len = sizeof(sktinfo->sa_family) + strlen(sktinfo->sa_data);

   debug("Binding to: %s", sktinfo->sa_data);
   // bind to the socket \& listen for connections
   assert (bind(*skt, sktinfo, len) >= 0, "Can't bind to address");   
   assert (listen(*skt, MAXCONNQUEUE) >= 0, "Can't listen on socket");
    
   chmod(sktinfo->sa_data, perms); // set permissions on the socket
}

@ 
@<Function...@>=
void setup_sockets(int* skt, struct sockaddr* sktinfo, char* file, mode_t perms);

@* Procedure |listen_sockets()|.
Loop, running |select(2)| over the sockets and handling the results from them.
@c
void listen_sockets()
{
   @<|listen_sockets| Local variables@>;
 
   for (;;) { /* Loop forever here unless told to exit by a control message */
      
      @<Add all the active sockets to an |fd_set|@>;
       
      timeout.tv_sec = 1;
      timeout.tv_usec = 0;

      assert ((rc = select(scount, &socks, NULL, NULL, &timeout)) >= 0, "Select Error");
      /* check for things to read */
      debug("Connections on %d sockets.", rc);

      debug("checking usb devices for interrupts");
      check_usb_interrupts();
       
      if (0 == rc) continue; /* nothing to read */

      {
         /* read from the remaining sockets. */
         if (FD_ISSET(master_socket, &socks)) /* the master socket */ { 
            @<Accept a new connection@>;
         }
 
         if (FD_ISSET(control_socket, &socks)) /* the control socket */ {
            if (handle_control_message()) return; // handle control message and exit if told to
         }

         
         current = client_root;
         while (NULL != current) /* the client sockets */ {
            if (FD_ISSET(current->socket, &socks)) {
               if (!service_client(current)) {
                  /* connection should be closed */
                  slog("Client connection (pid: %d) closed", current->pid);
                  close(current->socket); // close the connection
                  client_root = removeclient(client_root, current);
                     // remove the client from the list
                  free(current);
                  current = NULL;
               } else
                  current = current->next; // go onto the next client
            } else
               current = current->next; // go onto the next client
         }  
      }
   }
}

@ 
@<Function...@>=
void listen_sockets();

@ Variables local to |listen_sockets|.

@<|listen_sockets| Local...@>=
   socklen_t remote_len; // length of the sockaddr from the remote client
   int scount; // max($\{$fd $\mid$ fd $\in$ |fd_set|$\}$) + 1
   struct timeval timeout; // timeout to wait on select calls
   int rc = 0; // return value
   struct sockaddr tempaddr; // temporary variable
   int tempsock; // temporary variable
   int bytes; // number of bytes written to socket
   pclient newclient; // temporary variable
   pid_t remote_pid = 0; // PID read from the remote client
   pclient current; // temporary variable

@ Create a list of all active sockets.
We need to create an |fd_set| of all the active sockets.
Add the constant ones, and then delegate to |build_fd_list|
@<Add all the active sockets...@>=
      fd_set socks;
      FD_ZERO(&socks); // empty the FD\_SET
      FD_SET(master_socket, &socks); // add the master socket
      debug("master_socket: %d", master_socket);
      FD_SET(control_socket, &socks);
      debug("control_socket: %d", control_socket); // add the control socket
      scount = build_fd_list(&socks, client_root, max(control_socket, master_socket) + 1);
         // walk the list of client sockets
      
@ Accept a connection from a new client.
We need to call |accept(2)| on the |master_socket|, then update our linked list.
Also log their pid.

@<Accept a new connection@>= 
   tempsock = accept(master_socket, &tempaddr, &remote_len); // accept the connection
   if (0 != tempsock) {
      assert (tempsock >= 0, "Connection Failed");
 
      newclient = (pclient) malloc(sizeof(client));
      newclient->socket = tempsock;
      newclient->sockinfo = tempaddr;
      scount++; // create new client 
 
      assert ((bytes = recv(newclient->socket, &remote_pid, sizeof(pid_t), 0)) > 0, 
            "Receive Failed"); /* get pid */
      slog("New Client, PID: %d", remote_pid);
      newclient->pid = remote_pid;
      newclient->next = NULL;

      client_root = addclient(client_root,newclient); // add to linked list
   }

@* Function |handle_control_message()|.

Handles a message on the control socket and returns |true| if the daemon should exit.
@c
bool handle_control_message()
{
   ccid_request req;
   ccid_response res;
   int tempsock;
   struct sockaddr tempaddr;
   socklen_t remote_len;
   bool quit = false;
   debug("Handling Control Message");
 
   tempsock = accept(control_socket, &tempaddr, &remote_len); // accept a new socket on that socket
   assert (tempsock >= 0, "Connection Failed");
   
   assert (get_ccid_request(tempsock, &req) >=0, "Failed to receive on client socket");
   assert (PARAM == req.protocol, "Non-parameter message received on control socket");
      // get the request and check its protocol type
   
   @<Check the message contents and prepare the response structure@>;
   
   if (NULL != req.data) { free(req.data); req.data = NULL; } // free it if neccessary

   assert (send_ccid_response(tempsock, res) >=0, "Failed to send response to client");
      // return the response
 
   close(tempsock); // close the socket
   return quit;
}

@ 
@<Function...@>=
bool handle_control_message();

@ Checking the message sent to the daemon.
Currently we can be sent a quit message, or a message to re-read our config file.
This section also prepares an appropriate response to send to the other copy of the daemon.
@<Check the message...@>=
   res.protocol = PARAM;
   res.rdata = NULL;
   res.rsize = 0;

   /* check the message sent, and prepare the response structure */
   if (NULL == req.data) /* no message */ {
      slog("Empty control message, ignored", req.data);
      res.rdata = "Control message unknown";
      res.rsize = strlen(res.rdata) + 1;
   }

   else if (0 == strcmp(req.data, "quit")) /* quit message */ {
      slog("Got quit message, exitting");
      quit = true;
      res.rdata = "Daemon Quitting";
      res.rsize = strlen(res.rdata) + 1;
   }
   
   else if (0 == strcmp(req.data, "config")) /* re-read config message */ {
      slog("Got config message, re-reading config file");
      // TODO: re-read config file
      res.rdata = "Daemon re-read config file";
      res.rsize = strlen(res.rdata) + 1;
   }
   
   else /* unknown message */ {
      slog("Unknown control message '%s', ignored", req.data);
      res.rdata = "Control message unknown";
      res.rsize = strlen(res.rdata) + 1;
   }


@* Function |service_client()|.
Service a request by a client connection. We receive the request structure from the client,
then pass it down to the USB layer. The response is sent back to the client. Non-zero is
returned if the connection has been closed.
@c
int service_client(pclient cl)
{
   // TODO: detect wanting to close, and return $\neq$ 0
   ccid_request req;
   ccid_response res;
   debug("Servicing client %d", cl->pid);
   
   assert (get_ccid_request(cl->socket, &req) >=0, "Failed to receive on client socket");
      // read request from client
   
   debug("protocol: %d, datasize: %d, apdu type: %d", req.protocol, req.dsize, req.apdu.apdu_type);

   assert (usb_ccid_command(cl->cardref, &req, &res), "Failed to send command to client card");
      // send request to card and read response

   if (NULL != req.data) { free(req.data); req.data = NULL; }

   assert (send_ccid_response(cl->socket, res) >=0, "Failed to send response to client");
      // send response to client
   return 0;
}

@ 
@<Function...@>=
int service_client(pclient cl);
   
@* Procedure |clean_up()|.
Closes a socket and removes the file it was bound to

\halign{\hfil \it # & # \hfil \cr
|skt|& The file descriptor of the socket\cr
|sktinfo|& The info struct about the socket\cr
}
@c
void clean_up(int skt, struct sockaddr* sktinfo)
{
   close(skt);
   assert (unlink(sktinfo->sa_data) == 0, "Failed to clean up - couldn't unlink socket");
}

@ 
@<Function...@>=
void clean_up(int skt, struct sockaddr* sktinfo);

@* Procedure |check_running()|.

Checks if there is an already running copy.
If there isn't, then optionally fork from console
and write pid to pidfile. Returns |true| if a pidfile exists
@c
bool check_running(bool demonize)
{
   struct stat fs;
   pid_t pid;
   int ret;
   
   strcat(pidfile,SOCKETDIR);
   strcat(pidfile, "/pid");

   if (param.force) {
      // start anyway if the pidfile exists
      ret = unlink(pidfile);
      assert (ret >= 0 || errno == ENOENT, 
            "Failed to force removal of pidfile");
   }

   stat(pidfile, &fs); // stat the pidfile
   
   if (S_ISREG(fs.st_mode)) {
      // existing copy running
      return true;
   }
   pid = getpid();
   
   if (demonize) {
      @<Fork from console@>;
   }
   
   FILE* fp = fopen(pidfile, "w");
   assert (NULL != fp, "Couldn't write pidfile!");
   if (NULL == fp) exit(1);
   fprintf(fp, "%d", pid); // write pid to pidfile
   fclose(fp);
   return false;
}

@ 
@<Function...@>=
bool check_running(bool demonize);

@* Fork from console.
We have to |fork(2)| a child proccess and the exit the parent to return to the console,
then |setsid(2)| to disassociate ourselves from the tty we were spawned on. Finally,
we |chdir(2)| to the root directory so that filesystems can be unmounted.
@<Fork...@>=
   debug("Leaving console....");
   pid = fork();
   assert(pid >= 0, "Error in fork");
   if (0 < pid) exit(0);
   setsid();    
   chdir("/");
   umask(0);

@ Send message to another daemon.
Send our parameters to the existing copy, returning false if an error occurred.
@c
bool send_args()
{
   socklen_t len;
   struct sockaddr_struct info;

   int rc;
   int skt;

   ccid_request req;
   ccid_response res;
 
   skt = socket (AF_UNIX, SOCK_STREAM, 0); // get socket
   assert(skt > 0, "Can't get socket fd");
   info.family = AF_UNIX;
   strcpy(info.sa_data, SOCKETDIR);
   strcat(info.sa_data, "/control");
   len = sizeof(info.family) + strlen(info.sa_data) + 1;   
 
   debug("connecting to daemon");
   if ((rc = connect(skt, (struct sockaddr*) &info, len)) < 0) return false;

   req.protocol = PARAM;  // setup the request struct
   if (NULL == param.message) {
      req.data = NULL;
      req.dsize = 0;
   } else {
      req.data = param.message;
      req.dsize = strlen(param.message) + 1;
   }
   
   if ((rc = send_ccid_request(skt, req, &res)) < 0) return false;
      // send it

   if (NULL != res.rdata) /* print \& free the result */ {
      printf("%s\n", res.rdata);
      free(res.rdata);
      res.rdata = NULL;
   }

   close(skt); // close the socket

   return true;
}

@ 
@<Function...@>=
bool send_args();

