@** USB subsystem.
This file contains all the functions which interface with the USB directly.

@c
@<USB Header file includes@>@;
@<USB Static Variables@>@;
@<USB Internal Functions@>@;
@<USB Library Functions@>@;

@ Include header files.
We are using lib-USB to connect to the smart-cards, and also the shared
debug and protocol libraries. 
@<USB Header...@>=
#include <usb.h>
#include <errno.h>
#include "usb.h"
#include "structs.h"
#include "../share/types.h"
#include "../share/debug.h"
#include "../share/log.h"
#include "../share/protocol.h"

@* Exported interfaces.
These interfaces are exported in the header file {\tt usb.h}.
@(usb.h@>= 
#ifndef _CCID_USB_H
#define _CCID_USB_H
      
#include <usb.h>
#include "../share/types.h"
#include "../share/protocol.h"

#define SLOT_POWERED 0
#define SLOT_INSERTED 1
#define SLOT_EMPTY 2

#define USB_RW_TIMEOUT 6000
      
@<USB Function Prototypes@>@;

#endif

@* Function |initialise_usb()|.
Initialises the USB bus, and scans the busses for devices.
The function returns false if an error occurred.
@<USB Lib...@>=
bool initialise_usb()
{
   
   debug("Initialising USB library");
   usb_init(); /* Initialise the library code */
   if (!scan_usb()) return false; /* Scan for devices */

   return true;
}

@ Prototype.
@<USB Function Prototypes@>=
bool initialise_usb();

@* Function |scan_usb()|.
This function scans the USB bus for new devices, and removes old ones from the list
as the are connected / unplugged.
@<USB Lib...@>=
bool scan_usb()
{
   @<|scan_usb| local variables@>;
   @<Create USB spec to match against@>;
  
   @<Scan for all devices with match@>;

   @<Delete Match@>;
}

@ Prototype.
@<USB Function Prototypes@>=
bool scan_usb();

@ Create a USB Match spec.
To find devices you create a |usb_match_handle| structure, which
contains a list of characteristics of USB devices to match against.
Repeated called to |usb_find_device()| will then find all the devices
matching that specification. The parameters to |usb_create_match()| are
as follows:
\halign{\hfil \it # & # & # \hfil \cr
VENDOR & |-1| & (Any Device)\cr
PRODUCT & |-1| & (Any Device)\cr
CLASS & |USB_CLASS_CCID| & (CCID devices)\cr
SUBCLASS & |-1| & (Any Device)\cr
PROTOCOL & |-1| & (Any Device)\cr
}
@<Create USB spec...@>=
usb_create_match(&match, -1, -1, USB_CLASS_CCID, -1, -1);
@ 
@<|scan_usb| loc...@>=
usb_match_handle* match = NULL;

@ Scan for all the devices with match.
Now we scan the USB bus with repeated |usb_find_device()| calls 
to find all our devices.
@<Scan for all...@>=
dev = NULL;
while (usb_find_device(match, dev, &dev) >= 0) {
   
   slog("Found a device: %s\n", dev->filename); 
      /* Log that we found a device */
   
   @<Add device to structure@>;
   
   @<Print device info@>;
}
@ 
@<|scan_usb| loc...@>=
struct usb_device* dev;
int i;

@ Delete the match variable.
@<Delete Match@>=
usb_free_match(match);

@ Put the device details in a struct.
@<Add device...@>=
   if (dev->descriptor->bNumConfigurations != 1) {
      slog("Ignoring device: %s - non-singular configuration\n", 
            dev->filename);
      continue;
   }
   
   if (dev->config[0].bNumInterfaces != 1) {
      slog("Ignoring device: %s - non-singular interfaces\n", 
            dev->filename);
      continue;
   }
 
   if (dev->config[0].interface[0].num_altsetting != 1) {
      slog("Ignoring device: %s - non-singular alt-settings\n", 
            dev->filename);
      continue;
   }

   if (dev->config[0].interface[0].altsetting[0].extralen != 0x36) {
      slog("Ignoring device: %s - CCID extra descriptor length invalid\n", 
            dev->filename);
      continue;
   }

   usb_open(dev, &temp_handle); // get a file descriptor for the device.
   
   max_slots = dev->config[0].interface[0].altsetting[0].extra[4];
   for (; max_slots >=0; max_slots--) { // add a device entry for each slot
      temp = malloc(sizeof(struct device_struct));
      @<Setup new device structure@>;
      temp->slot = max_slots; // points to the current slot

      if (NULL == dev_root) {
         /* no devices yet, this one is the root */
         debug("Adding %s as root\n", dev->filename);
         slog("Adding device: %s", dev->filename);
         dev_root = temp;
      } else  for (current = dev_root ; ; current = current->next) { 
         /* search along list in case we have it */

         if (0 == strcmp(current->filename, dev->filename)) {
            /* We have already seen this device, so we exit this block
               and go onto the next one */
            debug("Already detected %s\n", dev->filename);
            free(temp);
            break; 
         }   

         if (NULL == current->next) {
            /* the end of the list, put it here and exit */
            debug("Adding %s to list\n", dev->filename);
            slog("Adding device: %s", dev->filename);
            current->next = temp;
            free(temp);
            break;
         }
      }

      if (SLOT_INSERTED == get_slot_status(temp)) 
         // check if there is already a card inserted
         device_turn_on(temp);
   }

@ 
@<|scan_usb| loc...@>=
pdevice temp;
pdevice current;
int max_slots;
struct usb_dev_handle* temp_handle;

@ Global device list.
We store a static global list of the devices.
@<USB Static Variables@>=
pdevice dev_root = NULL;

@ Setting up a device structure.
@<Setup new device...@>=
   temp->filename = dev->filename;
   temp->udev = dev;
   temp->handle = temp_handle; 
      // we open this first because we only open it once per device, not per slot
   temp->interface_number = dev->config[0].interface[0].
                            altsetting[0].bInterfaceNumber;
   temp->bulk_in = temp->bulk_out = temp->intr = -1;

   @<Read endpoints@>;
   @<Read extra data@>;
   temp->next = NULL;


@ Check endpoint types.
Here we have to check which endpoint is which. For bulk endpoints, |bmAttributes == 0x02|
and for interrupt endpoints |bmAttributes == 0x03|. A bulk-in endpoint has bit 7 of the 
address set to |1| (|bEndpointAddress & 0x80 != 0|) and the bulk-out endpoint has it set 
to |0|.
@<Read endpoints@>=
   for (i = 0; i < dev->config[0].interface[0].altsetting[0].bNumEndpoints; i++) {
      // check each endpoint for type
      if (0x02 == dev->config[0].interface[0].altsetting[0].endpoint[i].bmAttributes) {
         // its a bulk endpoint
         if (dev->config[0].interface[0].altsetting[0].endpoint[i].bEndpointAddress & 0x80) {
            // its a bulk-in endpoint
            temp->bulk_in = 
               dev->config[0].interface[0].altsetting[0].endpoint[i].bEndpointAddress; 
            temp->packet_size = 
               dev->config[0].interface[0].altsetting[0].endpoint[i].wMaxPacketSize; 
         } else
            // its a bulk-out endpoint
            temp->bulk_out = 
               dev->config[0].interface[0].altsetting[0].endpoint[i].bEndpointAddress;
      } else if (0x03 == dev->config[0].interface[0].altsetting[0].endpoint[i].bmAttributes)
         // its an interrupt endpoint.
         temp->intr = dev->config[0].interface[0].altsetting[0].endpoint[i].bEndpointAddress;
   }

@ Read CCID Extra Data.
The information on which voltages and protocols the card supports is stored in 
an extra class descriptor block.
@<Read extra...@>=
   temp->voltage = dev->config[0].interface[0].altsetting[0].extra[5];
   temp->protocols = dev->config[0].interface[0].altsetting[0].extra[6];
 

@* Turning on a device.
The function |device_turn_on()| is responsible for turning on a CCID card,
reading the Answer To Reset and putting all the data into the appropriate
structures.

@<USB Internal...@>=
bool device_turn_on(pdevice dev)
{
   @<Power Up Local Variables@>;
   
   slog("Powering up card %s.%d", dev->filename, dev->slot);
   dev->card = malloc(sizeof(card));
   @<Power up the card@>;
   @<Read the ATR@>;
}
   
@ Powering up the card.
To power up the card you need to send a CCID Reset command.
This is a ten (10) byte string containing certain values:

\halign{\hfil \it # & # & # & # \hfil \cr
{\bf Byte-indices }&{\bf Field }&{\bf Value }&{\bf Description }\cr
   0 & |bMessageType| & |0x62| &  This is a type-62 message \cr
   1-4 & |dwLength| & |0x0| &  No extra data \cr
   5 & |bSlot| & |0x0|-|0xff| &  The slot to power up\cr
   6 & |bSeq| & |0x0|-|0xff| & Sequence number for command?\cr
   7 & |bPowerSelect| & |0x0| & Select voltage automatically\cr
   8-9 & |abRFU| & |NULL| & Reserved for future use\cr
}

@<Power up the card@>=

#define SEQ 0
/* The |#define|s are temporary */

buf = malloc(10); // allocate a 10-byte string
buf[0] = 0x62; // |bMessageType|
buf[1] = buf[2] = buf[3] = buf[4] = 0x0; // |dwLength|
buf[5] = dev->slot; // |bSLot|
buf[6] = SEQ; // |bSeq|
if (dev->voltage & 0x01)
   buf[7] = 0x01; // |bPowerSelect|
else if (dev->voltage & 0x02)
   buf[7] = 0x02; // |bPowerSelect|
else if (dev->voltage & 0x04)
   buf[7] = 0x03; // |bPowerSelect|
debug("doing write..... %d\n",
      usb_bulk_write(dev->handle, dev->bulk_out, buf, 10, USB_RW_TIMEOUT));
free(buf);

@ 
@<Power Up Local Variables@>=
char* buf;

@ Read the ATR.
The Answer to Reset contains various data required to be able to communicate
with the card. Specifically it returns the protocol used by this card, T=0
or T=1. The ICC response is 10 bytes long, followed by |dwLength| bytes of the ATR.

The ICC response is:

\halign{\hfil \it # & # & # & # \hfil \cr
{\bf Byte-indices }&{\bf Field }&{\bf Value }&{\bf Description }\cr
0 & |bMessageType| & |0x80| & This is a message from the CCID device \cr
1-4 & |dwLength| & & The length of the extra data (The ATR data) \cr
5 & |bSlot| & & Corresponds to the power up message that caused this ATR. \cr
6 & |bSeq| & & Corresponds to the power up message that caused this ATR. \cr
7 & |bStatus| & & Slot status \cr
8 & |bError| & & Any errors \cr
9 & |bChainParameter| & |0x00| or |0x01| & |0x01| if this message has more parts \cr
}

The ATR contains the protocol to use (T=0 or T=1) as the low order 4 bits 
of the second byte of the ATR data. This is byte 11 in our read buffer.

@<Read the ATR@>=
buf = malloc(dev->packet_size);
debug("doing read..... (%d) %s\n",
      usb_bulk_read(dev->handle, dev->bulk_in, buf, 
                    dev->packet_size, USB_RW_TIMEOUT),
      usb_strerror());
debug("system error: (%d) %s\n", errno, strerror(errno));
debug("message: %d\nstatus: %d\nerror: %d\n",buf[0], buf[7], buf[8]);
length = (unsigned int) buf[1];
debug("ATR length: %d\n", length);
dev->card->protocol = buf[11] & 0x0F; // get out the protocol
free(buf);

@ 
@<Power Up Local Variables@>=
unsigned int length;

@* USB interrupt scanning.
We need to regularly check all the card reader devices for
messages on their interrupt lines.
@<USB Lib...@>=
void check_usb_interrupts()
{
}

@ Function Prototype.
@<USB Function Prototypes@>=
void check_usb_interrupts();

@* Function |get_slot_status()|.
This checks whether a slot has a card in it or not.
Returns the slot status, which is:

\halign{\hfil \it # & # & # \hfil \cr
   |SLOT_POWERED| & |0x0| & Card is inserted and powered up \cr
   |SLOT_INSERTED| & |0x1| & Card is inserted and not powered up \cr
   |SLOT_EMPTY| & |0x2| & Card not present \cr
}

The CCID slot status command is a ten (10) byte string containing certain values:

\halign{\hfil \it # & # & # & # \hfil \cr
   {\bf Byte-indices }&{\bf Field }&{\bf Value }&{\bf Description }\cr
   0 & |bMessageType| & |0x65| &  This is a type-65 message \cr
   1-4 & |dwLength| & |0x0| &  No extra data \cr
   5 & |bSlot| & |0x0|-|0xff| &  The slot to query\cr
   6 & |bSeq| & |0x0|-|0xff| & Sequence number for command?\cr
   7-9 & |abRFU| & |NULL| & Reserved for future use\cr
}

The response is also a 10 byte structure with no extra data:

\halign{\hfil \it # & # & # & # \hfil \cr
   {\bf Byte-indices }&{\bf Field }&{\bf Value }&{\bf Description }\cr
   0 & |bMessageType| & |0x81| & This is a SlotStatus message from the CCID device \cr
   1-4 & |dwLength| & |0x00| & The length of the extra data, always 0 \cr
   5 & |bSlot| & & Corresponds to the slot number in the request. \cr
   6 & |bSeq| & & Corresponds to the seq number in the request. \cr
   7 & |bStatus| & & Slot status \cr
   8 & |bError| & & Any errors \cr
   9 & |bClockStatus| & |0x00| - |0x03| & \cr
}

@<USB Inter...@>=
int get_slot_status(pdevice dev)
{
   #define SEQ 0
   /* The |#define|s are temporary */

   int status;

   char* buf;

   debug("Querying slot %d", dev->slot);
   
   { /* Send the Command */
      buf = malloc(10); // allocate a 10-byte string
      buf[0] = 0x65; // |bMessageType|
      buf[1] = buf[2] = buf[3] = buf[4] = 0x0; // |dwLength|
      buf[5] = dev->slot; // |bSLot|
      buf[6] = SEQ; // |bSeq|
      debug("doing write..... %d\n",
            usb_bulk_write(dev->handle, dev->bulk_out, buf, 10, USB_RW_TIMEOUT));
      free(buf);
   }

   { /* Read the response */ 
      buf = malloc(dev->packet_size); // allocate a 10-byte string
      debug("doing read..... (%d) %s\n",
            usb_bulk_read(dev->handle, dev->bulk_in, buf, 
               dev->packet_size, USB_RW_TIMEOUT),usb_strerror());
      status = buf[7] & 0x3; // only the first 2 bits;
      free(buf);
   }  

   return status;
}

@* Send a USB command to a device.
This function sends a |ccid_request| to a given device, and reads the 
|ccid_response| from it. The function returns false if an error occurred.
@<USB Lib...@>=
bool usb_ccid_command(usb_dev_handle* handle, ccid_request* req, ccid_response* res)
{
   debug("Doing cool USB stuff");
   res->protocol = req->protocol;
   res->rsize = 0;
   res->rdata = NULL;
   void* buffer = NULL;
   int length;

   @<Decode request into the buffer@>;
   
   // usb\_bulk\_write(handle, 0x02, buffer, length, 60000); // send req
   if (NULL != buffer) free(buffer);
   buffer = NULL;

   buffer = malloc(length);
   // length = usb\_bulk\_read(handle, 0x02, buffer, length, 6000); // get res

   @<Encode buffer as a response structure@>;
  
   free(buffer);
   buffer = NULL;
   return true;
}

@ Prototype.
@<USB Function Prototypes@>=
bool usb_ccid_command(usb_dev_handle* handle, ccid_request* req, ccid_response* res);

@ Decode a request into a buffer.
@<Decode...@>=
   switch (req->protocol) {
      case APDU:
         if (!decode_apdu_req(req, &buffer)) ;//TODO: error handling
         break;
      case TPDU:
      default:
         return false;
   }

@ Encode a buffer as a response.
@<Encode...@>=
   switch (req->protocol) {
      case APDU:
         if (!encode_apdu_res(buffer, res)) ;// TODO: error handling
         break;
      case TPDU:
      default:
         return false;
   }
 

@* Function |encode_apdu_res()|.
Turns a |void*| buffer into a |ccid_response| structure.
The function returns false if an error occurred.

NOTE: may |malloc(2)| |res->rdata|. If |res->rsize > 0| you {\bf MUST}
use |free(2)| to deallocate it when you are finished.

\halign{\hfil \it # & # \hfil \cr
|buffer|& The bit-stream to parse from the ISO protocol\cr
|res|& The structure to write the bit-stream to\cr
}
@<USB Int...@>=
bool encode_apdu_res(void* buffer, ccid_response* res)
{
   return true;
}

@* Function |decode_apdu_req()|.
Turns a |ccid_request| structure into |void*| buffer for writing to a USB device.
The function returns false if an error occurred.


NOTE: may |malloc(2)| |buffer|. If the function returns |true| you {\bf MUST}
use |free(2)| to deallocate it when you are finished.

\halign{\hfil \it # & # \hfil \cr
|buffer|& A pointer to a |void*| buffer\cr
|req|& The structure to decode to the buffer\cr
}
@<USB Int...@>=
bool decode_apdu_req(ccid_request* req, void** buffer)
{
   char header[4]; // [0] = CLA, [1] = INS, [2] = P1, [3] = P2
   header[0] = 0x00;
   header[1] = 0x00;
   switch (req->apdu.apdu_type) {
     case APDU_AUTHENTICATE:
        @<APDU authenticate decode@>;
     case APDU_BINARY:
        @<APDU binary decode@>;
     case APDU_CHALLENGE:
        @<APDU challenge decode@>;
     case APDU_CHANNEL:
        @<APDU channel decode@>;
     case APDU_DATA:
        @<APDU data decode@>;
     case APDU_RECORD:
        @<APDU record decode@>;
     case APDU_SELECT:
        @<APDU select decode@>;
     case APDU_VERIFY:
        @<APDU verify decode@>;
     default:
        return false;
   }
   return true;
}

@ Decoding an APDU Binary command.
@<APDU bina...@>=
  switch (req->apdu.binary.mode) {
     case READ: 
        if (0 == header[1]) header[1] = 0xB0;
        // set the INS to READ BINARY
     case ERASE: 
        if (0 == header[1]) header[1] = 0x0E;
        // set the INS to ERASE BINARY
        *buffer = malloc(3+4); // payload + header size
        ((char*) *buffer)[6] = (req->apdu.binary.length & 0xFF);
        ((char*) *buffer)[5] = ((req->apdu.binary.length & 0xFF00) >> 4);               
        ((char*) *buffer)[4] = 0;
        // Lc = null, Data = null, Le = |req->apdu.binary.length|
        // buffer[0] = 0 => extended format Le
        break;
     case WRITE: 
        if (0 == header[1]) header[1] = 0xD0;
        // set the INS to WRITE BINARY
     case UPDATE: 
        if (0 == header[1]) header[1] = 0xD6; 
        // set the INS to UPDATE BINARY
        *buffer = malloc((req->dsize & 0xFFFF) + 3 + 4); // payload + payloadsize + header
        ((char*) *buffer)[6] = (req->dsize & 0xFF);
        ((char*) *buffer)[5] = ((req->dsize & 0xFF00) >> 4);               
        ((char*) *buffer)[4] = 0;
        // Lc = |req->dsize|, Data = |req->data|, Le = null
        // |buffer[0] = 0| $\Rightarrow$ extended format Lc
        void* b = *buffer+7; // WTF?!
        memcpy(b, *buffer, req->dsize);
        break;
     case APPEND:
     default: return false;
  }
  header[3] = (req->apdu.binary.offset & 0xFF);
  header[2] = ((req->apdu.binary.offset & 0x7F00) >> 4);
  memcpy(*buffer, header, 4);
  break;

@* Procedure |usb_device_on()|.
Initialises a USB device and tells it to turn on.
@<USB Int...@>=
void usb_device_on(usb_dev_handle *handle)
{
  unsigned char buffer[1+256+2];
  int nlength;

  nlength = sizeof(buffer);

  unsigned char cmd[10];
  int retval;
  int length = 1;

  cmd[0] = 0x62; /* IccPowerOn */
  cmd[1] = cmd[2] = cmd[3] = cmd[4] = 0;  /* dwLength */
  cmd[5] = 0;     /* slot number */
  cmd[6] = 0x05;
  cmd[7] = 0x01;  /* 5.0V */
  cmd[8] = cmd[9] = 0; /* RFU */

  length=sizeof(cmd);
  retval = usb_bulk_write(handle, 0x02, cmd, length, 60000);

  nlength = usb_bulk_read(handle, 0x02, buffer, nlength, 6000);

}

