@** The Library used by client applications.

This file contains all the code linked to by applications wanting to access the 
smart cards. The associated header file contains all the functions exported to the 
client to communicate with a smart card. Programs using this may have to |free(2)| 
or |malloc(2)| some data buffers and should read the documentation below carefully
to know when this is their responsibility.

Communication over the socket to the daemon process is handled by the shared protocol 
layer, we merely make calls to that.

@* Exported interfaces.
These interfaces are exported via the file {\tt library.h}
@(library.h@>=
#ifndef _CCID_LIBRARY_H 
#define _CCID_LIBRARY_H

#include "../share/types.h"
#include <sys/socket.h>

@<Library Type Definitions@>@;

@<Exported Library Functions@>@;

#endif

@ System header files used.
We use string and error handling libraries
@c
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

@ Local header files.
These give us access to the protocol layer, debugging functions, and the exported function
declarations  from the library header file.
@c
#include "library.h"
#include "../share/protocol.h"
#include "../share/types.h"
#include "../share/debug.h"
#include "../share/log.h"

@* Function |get_card_reference()|.

This function allows you to request a connection to a smart card. 
You can specify the card to request in several different ways, some 
of which are blocking, and some are non-blocking.

{\bf Blocking requests:}

You can make a request for the next available smart card. 
This request will block until either a smart card is inserted,
or one in use by a different card is made available.

{\bf Non-Blocking requests:}

You can make a request which will return a card if there is one available,
but will return immediately with a |NOT_AVAIL| message if there are 
no free cards.

{\bf Specfiying cards}

Smart cards can be specified more precisely by giving a pattern to match describing them.
If you specify a file and contents, then any candidate card to be returned will be checked
to see if that file exists, and if the contents match the string given in the second parameter.
If |file == NULL|, then no checks will be performed. Otherwise, if |content == NULL| then the 
file will be checked for existance, but not for content.

{\bf Possible Requests}

\halign{\hfil \it # & # & # \hfil \cr
\bf Request & \bf Blocking & \bf Description \cr
|BLOCK_NEXT_AVAIL| & yes & Blocks until a card is available and returns a descriptor. \cr
|AVAIL| & no & Returns the error |NOT_AVAIL| if a card isn't available. This will not block \cr
}

@<Exported Library Functions@>=
int get_card_reference(card_ref* ref, CARD_REQUEST request, char* file, char* content);

@ |CARD_REQUEST| Type definition.
This type selects what type of request to make.
@<Library Type Definitions@>=
typedef enum { AVAIL, BLOCK_NEXT_AVAIL } CARD_REQUEST;

@ Card references.
A card reference contains the information about how to connect to one smart card. 
A card reference must be retrieved from a |get_card_reference()| call, and should
be relinquished with a |release_card()| call when finished with.

Since sockets are being used to identify cards in the cardd, this is merely a 
socket identifier and info block.

@<Library Type Definitions@>=
struct card_ref_struct {
   struct sockinfo_struct {
      sa_family_t family;
      char sockpath[MAXSOCKETPATH];
   } info;
   int socket;
};

typedef struct card_ref_struct card_ref;

@ Implementation of function.
@c
int get_card_reference(card_ref* ref, CARD_REQUEST request, char* file, char* content)
{
   socklen_t len;
   pid_t pid;
   int rc;

   debug("socket: %d", ref->socket);
   ref->socket = socket (AF_UNIX, SOCK_STREAM, 0);
   debug("getting socket, errno: (%d) %s", errno, strerror(errno));
   assert(ref->socket > 0, "Can't get socket fd");
   debug("socket: %d", ref->socket);

   ref->info.family = AF_UNIX;
   strcpy(ref->info.sockpath, SOCKETDIR);
   strcat(ref->info.sockpath, "/master");
   debug("master socket: %d, %s", ref->socket, ref->info.sockpath);
   len = sizeof(ref->info.family) + strlen(ref->info.sockpath) + 1;

   /* connect */
   debug("connecting to daemon");
   if ((rc = connect(ref->socket, (struct sockaddr*) &(ref->info), len)) < 0) return rc;
  
   /* send our pid */
   pid = getpid();
   debug("sending pid (%d)", pid);
   if ((rc = send(ref->socket, &pid, sizeof(pid_t), 0)) < 0) return rc;

   return 0;
}

@* Function |release_card()|.
Causes the cardd to close the connection to a smart card and make it available
for other applications. This also invalidates the |card_ref| block.

@<Exported Library Functions@>=
int release_card(card_ref ref);

@ Implementation of function.
@c
int release_card(card_ref ref)
{
   // TODO: something
   return 0;
}

@* Function |apdu_read_binary()|.
Sends an APDU READ BINARY command. We read |length| bytes from position |offset|
in the currently selected EF.
These are returned in the buffer |returneddata|. {\bf You must |malloc(2)| 
   |length| bytes of 
|returneddata|}. If no data is returned, then |returnedlength| will be |0|.

Binary commands can only be performed on EFs with transparent structure.

The return value from this function will be the APDU status that the card returns.

@<Exported Library Functions@>=
   
int apdu_read_binary(card_ref ref, int offset, int length, 
                     char* returneddata, int* returnedlength);

@ Implementation of function.   
@c
int apdu_read_binary(card_ref ref, int offset, int length, 
                     char* returneddata, int* returnedlength)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_BINARY;
   req.apdu.binary.mode = READ;
   req.apdu.binary.offset = offset;
   req.apdu.binary.length = length;
   req.dsize = 0;
   req.data = NULL;
   /* setup the request */
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   *returnedlength = res.rsize;
   if (NULL != res.rdata) {
      memcpy(res.rdata, returneddata, min(length, res.rsize));
      free(res.rdata);
      res.rdata = NULL;
   } else
      returnedlength = 0;
   
   return res.apdu.status;
}

@* Function |apdu_write_binary()|.
The WRITE BINARY command writes binary values to the currently selected EF.
Depending on the attributes of the file it will either OR the bits of the 
file with the bits supplied in the command, AND the two together, or simply
overwrite them. The |length| bytes specified by |data| will be written to
byte position |offset| in the currently selected EF.

Binary commands can only be performed on EFs with transparent structure.

The return value from this function will be the APDU status that the card returns.
@<Exported Library Functions@>=
   
int apdu_write_binary(card_ref ref, int offset, int length, char* data);

@ Implementation of function.   
@c
int apdu_write_binary(card_ref ref, int offset, int length, char* data)
{
   ccid_request req;
   ccid_response res;

   debug("apdu_write_binary");
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_BINARY;
   req.apdu.binary.mode = WRITE;
   req.apdu.binary.offset = offset;
   req.apdu.binary.length = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      free(res.rdata);
   } 
   
   return res.apdu.status;
}

@* Function |apdu_update_binary()|.
An UPDATE BINARY command updates the bytes from position |offset| in the
currently selected EF, with the bytes specified by |length| and |data|.

Binary commands can only be performed on EFs with transparent structure.

The return value from this function will be the APDU status that the card returns.
@<Exported Library Functions@>=

int apdu_update_binary(card_ref ref, int offset, int length, char* data);

@ Implementation of function.
@c
int apdu_update_binary(card_ref ref, int offset, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_BINARY;
   req.apdu.binary.mode = UPDATE;
   req.apdu.binary.offset = offset;
   req.apdu.binary.length = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      free(res.rdata);
   } 
   
   return res.apdu.status;
}


@* Function |apdu_erase_binary()|.
An ERASE BINARY command will set |length| bytes from |offset| in the
currently selected EF to the logically erased state.

Binary commands can only be performed on EFs with transparent structure.

The return value from this function will be the APDU status that the card returns.
@<Exported Library Functions@>=

int apdu_erase_binary(card_ref ref, int offset, int length);

@ Implementation of function.
@c
int apdu_erase_binary(card_ref ref, int offset, int length)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_BINARY;
   req.apdu.binary.mode = WRITE;
   req.apdu.binary.offset = offset;
   req.apdu.binary.length = length;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      free(res.rdata);
   } 
   
   return res.apdu.status;
}

@* Function |apdu_read_records()|.
This will read records from an EF. If |ef| is set to |CURRENT_EF| then
records will be read from the currently selected EF. Otherwise they will
be read from the specified EF. The record to start reading from are 
specified by |type| and |record|. 
If |type| $\in \{$|FIRST|,|LAST|,|NEXT|,|PREVIOUS|$\}$ 
then the appropriate record will be read from. If |type == SPECIFY|, then
the record number will be read from |record|. |length| bytes of data are 
read from the starting record.

{\bf You must |malloc(2)| |length| bytes of |returneddata|}.
If no data is returned, then |returnedlength| will be |0|.

The return value from this function will be the APDU status that the card returns.
@<Exported Library Functions@>=

int apdu_read_records(card_ref ref, int ef, RECORD_TYPE type,
                      int record, int length, int* returnedlength, char* returndata);

@ Implementation of function.
@c
int apdu_read_records(card_ref ref, int ef, RECORD_TYPE type,
                      int record, int length, int* returnedlength, char* returndata)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_RECORD;
   req.apdu.record.mode = READ;
   req.apdu.record.ef = ef;
   req.apdu.record.type = type;
   req.apdu.record.record = record;
   req.apdu.record.length = 0;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   returnedlength = res.rsize;
   if (NULL != res.rdata) {
      memcpy(res.rdata, returndata, min(length, res.rsize));
      free(res.rdata);
   } else
      returnedlength = 0;
   
   return res.apdu.status;
}

@* Function |apdu_write_record()|.
This will write records to an EF. If |ef| is set to |CURRENT_EF| then
records will be written to the currently selected EF. Otherwise they will
be written to the specified EF. Depending on the attributes of the file it 
will either OR the bits of the file with the bits supplied in the command,
AND the two together, or simply overwrite them. The record to start writing
to from are specified by |type| and |record|. 
If |type| $\in \{$|FIRST|,|LAST|,|NEXT|,|PREVIOUS|$\}$ 
then the appropriate record will be written to. If |type == SPECIFY|, then
the record number will be read from |record|. |length| bytes of data are
written from that point.

The return value from this function will be the APDU status that the card returns.
@<Exported Library Functions@>=

int apdu_write_record(card_ref ref, int ef, RECORD_TYPE type, int record, int length, char* data);

@ Implementation of function.
@c
int apdu_write_record(card_ref ref, int ef, RECORD_TYPE type, int record, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_RECORD;
   req.apdu.record.mode = WRITE;
   req.apdu.record.ef = ef;
   req.apdu.record.type = type;
   req.apdu.record.record = record;
   req.apdu.record.length = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


@* Function |apdu_append_record()|.
This will append records to an EF. If |ef| is set to |CURRENT_EF| then
records will be appended to the end of the currently selected EF. 
Otherwise they will be appended to the specified EF.

If the EF is a cyclic structure, then it will write to record 1.

The record pointer is set to the appended record.
@<Exported Library Functions@>=

int apdu_append_record(card_ref ref, int ef, int length, char* data);

@ Implementation of function.
@c
int apdu_append_record(card_ref ref, int ef, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_RECORD;
   req.apdu.record.mode = APPEND;
   req.apdu.record.ef = ef;
   req.apdu.record.length = 0;
   req.apdu.record.record = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


@* Function |apdu_update_record()|.
This will write records to an EF. If |ef| is set to |CURRENT_EF| then
records will be written to the currently selected EF. Otherwise they will
be written to the specified EF. Depending on the attributes of the file it
will either OR the bits of the file with the bits supplied in the command,
   AND the two together, or simply overwrite them. The record to start writing
to from are specified by |type| and |record|. 
If |type| $\in \{$|FIRST|,|LAST|,|NEXT|,|PREVIOUS|$\}$ 
then the appropriate record will be written to. If |type == SPECIFY|, then
the record number will be read from |record|. |length| bytes of data are
written from that point.

The return value from this function will be the APDU status that the card returns.

@<Exported Library Functions@>=

int apdu_update_record(card_ref ref, int ef, RECORD_TYPE type, int record, int length, char* data);

@ Implementation of function.
@c
int apdu_update_record(card_ref ref, int ef, RECORD_TYPE type, int record, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_RECORD;
   req.apdu.record.mode = UPDATE;
   req.apdu.record.ef = ef;
   req.apdu.record.type = type;
   req.apdu.record.record = record;
   req.apdu.record.length = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


@* Function |apdu_get_data()|.
{\bf You must |malloc(2)| |responselength| bytes of |responsedata|}.
@<Exported Library Functions@>=

int apdu_get_data(card_ref ref, TAG_TYPE tagtype, int tag, int responselength, char* responsedata);

@ Implementation of function.
@c
int apdu_get_data(card_ref ref, TAG_TYPE tagtype, int tag, int responselength, char* responsedata)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_DATA;
   req.apdu.data.mode = READ;
   req.apdu.data.tagtype = tagtype;
   req.apdu.data.tag = tag;
   req.apdu.data.length = responselength;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      memcpy(res.rdata, responsedata, min(responselength, res.rsize));
      free(res.rdata);
   }
   else responsedata = NULL;
   
   return res.apdu.status;
}

@* Function |apdu_put_data()|.
@<Exported Library Functions@>=

int apdu_put_data(card_ref ref, TAG_TYPE tagtype, int tag, int length, char* data);

@ Implementation of function.
@c
int apdu_put_data(card_ref ref, TAG_TYPE tagtype, int tag, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_DATA;
   req.apdu.data.mode = WRITE;
   req.apdu.data.tagtype = tagtype;
   req.apdu.data.tag = tag;
   req.apdu.data.length = 0;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


@* Function |apdu_select_file_id()|.
{\bf You must |malloc(2)| |responselength| bytes of |responsedata|}.
@<Exported Library Functions@>=

int apdu_select_file_id(card_ref ref, FILE_TYPE type, int DF, 
                        int responsetemplate, int responselength, char* responsedata);

@ Implementation of function.
@c
int apdu_select_file_id(card_ref ref, FILE_TYPE type, int DF, 
                        int responsetemplate, int responselength, char* responsedata)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_SELECT;
   req.apdu.select.mode = ID;
   req.apdu.select.rtemplate = responsetemplate;
   req.apdu.select.rsize = responselength;
   req.apdu.select.DF = DF;
   req.apdu.select.type = type;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      memcpy(res.rdata, responsedata, min(res.rsize, responselength));
      free(res.rdata);
   } else responsedata = NULL;
   
   return res.apdu.status;
}


@* Function |apdu_select_file_path()|.
{\bf You must |malloc(2)| |responselength| bytes of |responsedata|}.
@<Exported Library Functions@>=

int apdu_select_file_path(card_ref ref, bool relative, char* path, 
                          int responsetemplate, int responselength, char* responsedata);

@ Implementation of function.
@c
int apdu_select_file_path(card_ref ref, bool relative, char* path, 
                          int responsetemplate, int responselength, char* responsedata)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_SELECT;
   req.apdu.select.mode = PATH;
   req.apdu.select.rtemplate = responsetemplate;
   req.apdu.select.rsize = responselength;
   req.apdu.select.relative = relative;
   req.dsize = strlen(path)+1;
   req.data = path;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      memcpy(res.rdata, responsedata, min(res.rsize, responselength));
      free(res.rdata);
   } else responsedata = NULL;
   
   return res.apdu.status;
}


@* Function |apdu_select_file_df()|.
{\bf You must |malloc(2)| |responselength| bytes of |responsedata|}.
@c
int apdu_select_file_df(card_ref ref, char* name, int responsetemplate, 
                        int responselength, char* responsedata)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_SELECT;
   req.apdu.select.mode = DF;
   req.apdu.select.rtemplate = responsetemplate;
   req.apdu.select.rsize = responselength;
   req.dsize = strlen(name)+1;
   req.data = name;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      memcpy(res.rdata, responsedata, min(res.rsize, responselength));
      free(res.rdata);
   } else responsedata = NULL;
   
   return res.apdu.status;
}


@* Function |apdu_verify()|.
@<Exported Library Functions@>=

int apdu_verify(card_ref ref, bool global, int reference, int length, char* data);

@ Implementation of function.
@c
int apdu_verify(card_ref ref, bool global, int reference, int length, char* data)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_VERIFY;
   req.apdu.verify.global = global;
   req.apdu.verify.reference = reference;
   req.dsize = length;
   req.data = data;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


@* Function |apdu_authenticate_internal()|.
{\bf You must |malloc(2)| |responselength| bytes of |response|}.
@<Exported Library Functions@>=

int apdu_authenticate_internal(card_ref ref, bool globalsecret, int algorithmid, 
                              int secretid, int challengelength, char* challenge,
                              int responselength, char* response);

@ Implementation of function.
@c
int apdu_authenticate_internal(card_ref ref, bool globalsecret, int algorithmid, 
                              int secretid, int challengelength, char* challenge,
                              int responselength, char* response)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_AUTHENTICATE;
   req.apdu.auth.internal = true;
   req.apdu.auth.globalsecret = globalsecret;
   req.apdu.auth.algorithmid = algorithmid;
   req.apdu.auth.secretid = secretid;
   req.apdu.auth.responselength = responselength;
   req.dsize = challengelength;
   req.data = challenge;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      memcpy(res.rdata, response, min(res.rsize, responselength));
      free(res.rdata);
   }
   else response = NULL;
   
   return res.apdu.status;
}

@* Function |apdu_authenticate_external()|.
@<Exported Library Functions@>=

int apdu_authenticate_external(card_ref ref, bool globalsecret, int algorithmid, 
                               int secretid, int responselength, char* response);

@ Implementation of function.
@c
int apdu_authenticate_external(card_ref ref, bool globalsecret, int algorithmid, 
                               int secretid, int responselength, char* response)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_AUTHENTICATE;
   req.apdu.auth.internal = false;
   req.apdu.auth.globalsecret = globalsecret;
   req.apdu.auth.algorithmid = algorithmid;
   req.apdu.auth.secretid = secretid;
   req.apdu.auth.responselength = 0;
   req.dsize = responselength;
   req.data = response;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}

@* Function |apdu_get_challenge()|.
{\bf You must |malloc(2)| |maxlength| bytes of |challengereturned|}.
@<Exported Library Functions@>=

int apdu_get_challenge(card_ref ref, int maxlength, int* returnedlength, char* challengereturned);

@ Implementation of function.
@c
int apdu_get_challenge(card_ref ref, int maxlength, int* returnedlength, char* challengereturned)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_CHALLENGE;
   req.apdu.challenge.maxlength = maxlength;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   *returnedlength = res.rsize;
   if (NULL != res.rdata) {
      memcpy(res.rdata, challengereturned, min(maxlength, res.rsize));
      free(res.rdata);
   }
   else memset(challengereturned, 0, maxlength);
   
   return res.apdu.status;
}


@* Function |apdu_channel_open()|.
@<Exported Library Functions@>=

int apdu_channel_open(card_ref ref, int* channelno);

@ Implementation of function.
@c
int apdu_channel_open(card_ref ref, int* channelno)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_CHANNEL;
   req.apdu.channel.open = true;
   req.apdu.channel.channelno = 0;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata) {
      *channelno = *(int*) res.rdata;
      free(res.rdata);
   } else channelno = 0;
   
   return res.apdu.status;
}


@* Function |apdu_channel_close()|.
@<Exported Library Functions@>=

int apdu_channel_close(card_ref ref, int channelno);

@ Implementation of function.
@c
int apdu_channel_close(card_ref ref, int channelno)
{
   ccid_request req;
   ccid_response res;
   
   req.protocol = APDU;
   req.apdu.apdu_type = APDU_CHANNEL;
   req.apdu.channel.open = false;
   req.apdu.channel.channelno = channelno;
   req.dsize = 0;
   req.data = NULL;
   
   if (!send_ccid_request(ref.socket, req, &res)) ;// TODO: error type stuff
   
   if (APDU != res.protocol) ;// TODO: error type stuff 
   
   if (NULL != res.rdata)
      free(res.rdata);
   
   return res.apdu.status;
}


