@** Logging functions.

This file contains functions for sending log messages to syslog, and to perform 
conditional logging.

@ Header files and global variables.
We need syslog header files, and also debugging code.
We store a static global variable which governs whether to use syslog or to print
messages on |stderr|.
@c
#include <syslog.h>
#include <stdarg.h>
#include <errno.h>
#include "log.h"
#include "debug.h"

bool _do_syslog = false;

@* Exported interfaces.
The file {\tt log.h} exports several macros and functions to other files.
@(log.h@>=

#ifndef __CCID_LOG_H
#define __CCID_LOG_H

#include "types.h"

#ifdef DEBUG
#define assert(a, b) do { bool __T=a; \
                          fprintf(stderr,"[Assert] %s\n", __T?"true":"false"); \
                          if(!__T) real_assert(b);\
                        } while(0)
#else
#define assert(a, b) if(!(a)) real_assert(b)
#endif

void real_assert (char* message);

void slog(char* fmt, ...);

void setup_syslog(bool log);

#endif

@* Procedure |real_assert()|.
@c
void real_assert (char* message)
{
   if (0 != errno) {
      char* errstr = strerror(errno);
      slog("[Assert] failed with errno %d/%s: %s\n", errno, errstr, message);
   } else
      slog("[Assert] failed: %s\n", message);
#ifdef DEBUG
   exit(1);
#endif
}

@* Procedure |slog()|.
@c
void slog(char* fmt, ...)
{
   va_list args;

   va_start(args, fmt);
   
   if (_do_syslog)
      vsyslog(LOG_NOTICE, fmt, args);
   else {
      fprintf(stderr, "[ccid-cardd] ");
      vfprintf(stderr, fmt, args);
      fprintf(stderr, "\n");
   }

   va_end(args);
}

@* Procedure |setup_syslog()|.
@c
void setup_syslog(bool log)
{
   _do_syslog = log;
   if (log)
      openlog("ccid-cardd",0, LOG_DAEMON);
}


