/*
 * Main API entry point
 *
 * Copyright (c) 2000-2002 Johannes Erdfelt <johannes@erdfelt.com>
 *
 * This library is covered by the LGPL, read LICENSE for details.
 */

#include <stdlib.h>	/* getenv */
#include <stdio.h>	/* stderr */
#include <errno.h>

#include "usbi.h"

#define USB_DEFAULT_DEBUG_LEVEL		0
int usb_debug = USB_DEFAULT_DEBUG_LEVEL;
static struct usbi_bus *usb_busses = NULL;

int usb_rescan_devices(void)
{
  struct usbi_bus *busses, *ibus;
  int ret, changes = 0;

  ret = usb_os_find_busses(&busses);
  if (ret < 0)
    return ret;

  /*
   * Now walk through all of the busses we know about and compare against
   * this new list. Any duplicates will be removed from the new list.
   * If we don't find it in the new list, the bus was removed. Any
   * busses still in the new list, are new to us.
   */
  ibus = usb_busses;
  while (ibus) {
    int found = 0;
    struct usbi_bus *nibus, *tibus = ibus->next;

    nibus = busses;
    while (nibus) {
      struct usbi_bus *tnibus = nibus->next;

      if (!strcmp(ibus->filename, nibus->filename)) {
        /* Remove it from the new devices list */
        LIST_DEL(busses, nibus);

        usb_free_bus(nibus);
        found = 1;
        break;
      }

      nibus = tnibus;
    }

    if (!found) {
      /* The device was removed from the system */
      LIST_DEL(usb_busses, ibus);
      changes++;
    }

    ibus = tibus;
  }

  /* Anything on the *busses list is new. So add them to usb_busses */
  /*  and process them like the new bus they are */
  ibus = busses;
  while (ibus) {
    struct usbi_bus *tibus = ibus->next;

    /* Remove it from the temporary list first and add it to the real */
    /*  usb_busses list */
    LIST_DEL(busses, ibus);

    LIST_ADD(usb_busses, ibus);

    changes++;

    ibus = tibus;
  }

  for (ibus = usb_busses; ibus; ibus = ibus->next) {
    struct usbi_device *devices, *idev;

    /* Find all of the devices and put them into a temporary list */
    ret = usb_os_find_devices(ibus, &devices);
    if (ret < 0)
      return ret;

    /*
     * Now walk through all of the devices we know about and compare
     * against this new list. Any duplicates will be removed from the new
     * list. If we don't find it in the new list, the device was removed.
     * Any devices still in the new list, are new to us.
     */
    idev = ibus->devices;
    while (idev) {
      int found = 0;
      struct usbi_device *nidev, *tidev = idev->next;

      nidev = devices;
      while (nidev) {
        struct usbi_device *tnidev = nidev->next;

        if (!strcmp(idev->filename, nidev->filename)) {
          /* Remove it from the new devices list */
          LIST_DEL(devices, nidev);

          usb_free_dev(nidev);
          found = 1;
          break;
        }

        nidev = tnidev;
      }

      if (!found) {
        /* The device was removed from the system */
        LIST_DEL(ibus->devices, idev);
        changes++;
      }

      idev = tidev;
    }

    /* Anything on the *devices list is new. So add them to bus->devices */
    /*  and process them like the new device they are */
    idev = devices;
    while (idev) {
      struct usbi_device *tidev = idev->next;

      /* Remove it from the temporary list first and add it to the real */
      /*  bus->devices list */
      LIST_DEL(devices, idev);

      LIST_ADD(ibus->devices, idev);

      /* Some ports fetch the descriptors on scanning (like Linux) so we */
      /*  don't need to fetch them again */
      if (!idev->config) {
        usb_dev_handle *udev;

        ret = usb_open(idev->dev, &udev);
        if (ret >= 0) {
          usb_fetch_and_parse_descriptors(udev);

          usb_close(udev);
        }
      }

      idev->dev = malloc(sizeof(*idev->dev));
      /* FIXME: What to do in the failure case here? */
      if (idev->dev) {
        idev->dev->idev = idev;
        idev->dev->filename = idev->filename;
        idev->dev->descriptor = &idev->descriptor;
        idev->dev->config = idev->config;
      }

      changes++;

      idev = tidev;
    }
  }

  return changes;
}

int usb_create_match(usb_match_handle **handle, int idVendor, int idProduct, int bClass, int bSubClass, int bProtocol)
{
  struct usb_match *match;

  match = malloc(sizeof(*match));
  if (!match)
    return -ENOMEM;

  match->idVendor = idVendor;
  match->idProduct = idProduct;
  match->bClass = bClass;
  match->bSubClass = bSubClass;
  match->bProtocol = bProtocol;

  *handle = match;

  return 0;
}

int usb_free_match(usb_match_handle *handle)
{
  free(handle);
}

static int usb_match_interfaces(struct usb_match *match, struct usbi_device *idev)
{
  int c;

  if (match->bClass < 0 && match->bSubClass < 0 && match->bProtocol < 0)
    return 0;

  /* Now check all of the configs/interfaces/altsettings */
  for (c = 0; c < idev->descriptor.bNumConfigurations; c++) {
    struct usb_config_descriptor *config = &idev->config[c];
    int i;

    for (i = 0; i < config->bNumInterfaces; i++) {
      struct usb_interface *intf = &config->interface[i];
      int a;

      for (a = 0; a < intf->num_altsetting; a++) {
        struct usb_interface_descriptor *altsetting = &intf->altsetting[a];

        if ((match->bClass < 0 || match->bClass == altsetting->bInterfaceClass) &&
            (match->bSubClass < 0 || match->bSubClass == altsetting->bInterfaceSubClass) &&
            (match->bProtocol < 0 || match->bProtocol == altsetting->bInterfaceProtocol))
          return 1;
      }
    }
  }

  return 0;
}

int usb_find_device(usb_match_handle *handle, struct usb_device *pdev, struct usb_device **dev)
{
  struct usb_match *match = handle;
  struct usbi_device *idev;
  struct usbi_bus *ibus;

  if (pdev) {
    /* Start on the next device */
    idev = pdev->idev;
    ibus = idev->bus;
    idev = idev->next;
  } else {
    /* Start on the first bus and device */
    if (!usb_busses)
      return -ENODEV;

    ibus = usb_busses;
    idev = ibus->devices;
  }

  while (1) {
    while (!idev) {
      /* Go to the next bus */
      ibus = ibus->next;
      if (!ibus)
        return -ENODEV;

      idev = ibus->devices;
    }

    for (; idev; idev = idev->next) {
      struct usb_device_descriptor *desc = &idev->descriptor;

      /* If we have no match criteria, then we're done */
      if (match) {
        int c;

        if ((match->idVendor >= 0 && match->idVendor != desc->idVendor) ||
            (match->idProduct >= 0 && match->idProduct != desc->idProduct))
          continue;

        if (!usb_match_interfaces(match, idev))
          continue;
      }

      *dev = idev->dev;
      return 0;
    }
  }
}

void usb_set_debug(int level)
{
  if (usb_debug || level)
    fprintf(stderr, "usb_set_debug: Setting debugging level to %d (%s)\n",
	level, level ? "on" : "off");

  usb_debug = level;
}

void usb_init(void)
{
  if (getenv("USB_DEBUG") && usb_debug == USB_DEFAULT_DEBUG_LEVEL)
    usb_set_debug(atoi(getenv("USB_DEBUG")));

  usb_os_init();
  usb_rescan_devices();
}

int usb_open(struct usb_device *dev, usb_dev_handle **handle)
{
  struct usbi_device *idev = dev->idev;
  usb_dev_handle *udev;
  int ret;

  udev = malloc(sizeof(*udev));
  if (!udev)
    return -ENOMEM;

  udev->fd = -1;
  udev->device = idev;
  udev->bus = idev->bus;
  udev->config = udev->interface = udev->altsetting = -1;

  ret = usb_os_open(udev);
  if (ret < 0) {
    free(udev);
    return ret;
  }

  *handle = udev;

  return 0;
}

int usb_get_string(usb_dev_handle *dev, int index, int langid, char *buf, size_t buflen)
{
  return usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
                        (USB_DT_STRING << 8) + index, langid, buf, buflen, 1000);
}

int usb_get_string_simple(usb_dev_handle *dev, int index, char *buf, size_t buflen)
{
  char tbuf[256];
  int ret, langid, si, di;

  /*
   * Asking for the zero'th index is special - it returns a string
   * descriptor that contains all the language IDs supported by the
   * device. Typically there aren't many - often only one. The
   * language IDs are 16 bit numbers, and they start at the third byte
   * in the descriptor. See USB 2.0 specification, section 9.6.7, for
   * more information on this. */
  ret = usb_get_string(dev, 0, 0, tbuf, sizeof(tbuf));
  if (ret < 0)
    return ret;

  if (ret < 4)
    return -EIO;

  langid = tbuf[2] | (tbuf[3] << 8);

  ret = usb_get_string(dev, index, langid, tbuf, sizeof(tbuf));
  if (ret < 0)
    return ret;

  if (tbuf[1] != USB_DT_STRING)
    return -EBADE;

  if (tbuf[0] > ret)
    return -EOVERFLOW;

  for (di = 0, si = 2; si < tbuf[0]; si += 2) {
    if (di >= (buflen - 1))
      break;

    if (tbuf[si + 1])   /* high byte */
      buf[di++] = '?';
    else
      buf[di++] = tbuf[si];
  }

  buf[di] = 0;

  return di;
}

int usb_close(usb_dev_handle *udev)
{
  int ret;

  ret = usb_os_close(udev);
  free(udev);

  return ret;
}

void usb_free_dev(struct usbi_device *idev)
{
  usb_destroy_configuration(idev->dev);
  free(idev->dev);
  free(idev);
}

void usb_free_bus(struct usbi_bus *ibus)
{
  free(ibus);
}

